@@ -59,23 +59,23 @@ html_font_set_face (HTMLFontSet *set, gchar *face)
 }
 
 static void
-html_font_set_release (HTMLFontSet *set, HTMLPainterClass *painter_class)
+html_font_set_release (HTMLFontSet *set, HTMLPainter *painter)
 {
 	gint i;
 
 	for (i=0; i<GTK_HTML_FONT_STYLE_MAX_FONT; i++) {
 		if (set->font [i])
-			html_painter_unref_font (painter_class, set->font [i]);
+			html_painter_unref_font (painter, set->font [i]);
 		set->font [i] = NULL;
 	}
 }
 
 static void
-html_font_set_unref (HTMLFontSet *set, HTMLPainterClass *painter_class)
+html_font_set_unref (HTMLFontSet *set, HTMLPainter *painter)
 {
 	set->ref_count --;
 	if (!set->ref_count) {
-		html_font_set_release (set, painter_class);
+		html_font_set_release (set, painter);
 		if (set->face)
 			g_free (set->face);
 
@@ -84,7 +84,7 @@ html_font_set_unref (HTMLFontSet *set, HTMLPainterClass *painter_class)
 }
 
 void
-html_font_manager_init (HTMLFontManager *manager, HTMLPainterClass *painter_class)
+html_font_manager_init (HTMLFontManager *manager, HTMLPainter *painter)
 {
 	manager->font_sets     = g_hash_table_new (g_str_hash, g_str_equal);
 	manager->var_size      = 12;
@@ -92,30 +92,12 @@ html_font_manager_init (HTMLFontManager *manager, HTMLPainterClass *painter_clas
 	manager->fix_size      = 12;
 	manager->fix_points    = FALSE;
 	manager->magnification = 1.0;
-	manager->painter_class = painter_class;
+	manager->painter       = painter;
 
 	html_font_set_init (&manager->variable, NULL);
 	html_font_set_init (&manager->fixed, NULL);
 }
 
-HTMLFontManager *
-html_font_manager_new (HTMLPainterClass *pc)
-{
-	HTMLFontManager *fm;
-
-	fm = g_new0 (HTMLFontManager, 1);
-	html_font_manager_init (fm, pc);
-
-	return fm;
-}
-
-void
-html_font_manager_destroy (HTMLFontManager *manager)
-{
-	html_font_manager_finalize (manager);
-	g_free (manager);
-}
-
 void
 html_font_manager_set_magnification (HTMLFontManager *manager, gdouble magnification)
 {
@@ -131,7 +113,7 @@ static gboolean
 destroy_font_set_foreach (gpointer key, gpointer font_set, gpointer data)
 {
 	g_free (key);
-	html_font_set_unref (font_set, HTML_PAINTER_CLASS (data));
+	html_font_set_unref (font_set, HTML_PAINTER (data));
 
 	return TRUE;
 }
@@ -139,24 +121,22 @@ destroy_font_set_foreach (gpointer key, gpointer font_set, gpointer data)
 static void
 clear_additional_font_sets (HTMLFontManager *manager)
 {
-	g_hash_table_foreach_remove (manager->font_sets, destroy_font_set_foreach, manager->painter_class);	
+	g_hash_table_foreach_remove (manager->font_sets, destroy_font_set_foreach, manager->painter);	
 }
 
 void
 html_font_manager_clear_font_cache (HTMLFontManager *manager)
 {
-	/* printf ("html_font_manager_clear_font_cache\n"); */
-
-	html_font_set_release (&manager->variable, manager->painter_class);
-	html_font_set_release (&manager->fixed, manager->painter_class);
+	html_font_set_release (&manager->variable, manager->painter);
+	html_font_set_release (&manager->fixed, manager->painter);
 	clear_additional_font_sets (manager);
 }
 
 void
 html_font_manager_finalize (HTMLFontManager *manager)
 {
-	html_font_set_release (&manager->variable, manager->painter_class);
-	html_font_set_release (&manager->fixed, manager->painter_class);
+	html_font_set_release (&manager->variable, manager->painter);
+	html_font_set_release (&manager->fixed, manager->painter);
 	g_free (manager->fixed.face);
 	g_free (manager->variable.face);
 
@@ -179,7 +159,7 @@ html_font_manager_set_default (HTMLFontManager *manager, gchar *variable, gchar
 		changed = TRUE;
 	}
 	if (changed) {
-		html_font_set_release (&manager->variable, manager->painter_class);
+		html_font_set_release (&manager->variable, manager->painter);
 	}
 	changed = FALSE;
 
@@ -190,16 +170,8 @@ html_font_manager_set_default (HTMLFontManager *manager, gchar *variable, gchar
 		manager->fix_points = fix_points;
 		changed = TRUE;
 	}
-	if (changed) {
-		HTMLFontManager *plain_fm = html_engine_class_plain_font_manager ();
-
-		html_font_set_release (&manager->fixed, manager->painter_class);
-
-		/* release plain fontmanager fonts as well as they are
-		   referenced gdk fontmanager ones */
-		html_font_set_release (&plain_fm->variable, manager->painter_class);
-		html_font_set_release (&plain_fm->fixed, manager->painter_class);
-	}
+	if (changed)
+		html_font_set_release (&manager->fixed, manager->painter);
 }
 
 static gint
@@ -230,7 +202,7 @@ get_real_font_size (HTMLFontManager *manager, GtkHTMLFontStyle style)
 	gint size = (get_font_num (style) & GTK_HTML_FONT_STYLE_SIZE_MASK) -  GTK_HTML_FONT_STYLE_SIZE_3;
 	gint base_size = style & GTK_HTML_FONT_STYLE_FIXED ? manager->fix_size : manager->var_size;
 
-	return MAX (4, manager->magnification * (base_size + (size > 0 ? (1 << size) : size) * base_size/8.0));
+	return manager->magnification * (base_size + (size > 0 ? (1 << size) : size) * base_size/8.0);
 }
 
 static void
@@ -244,7 +216,7 @@ html_font_set_font (HTMLFontManager *manager, HTMLFontSet *set, GtkHTMLFontStyle
 	/* set font in font set */
 	idx = html_font_set_get_idx (style);
 	if (set->font [idx] && font != set->font [idx])
-		html_painter_unref_font (manager->painter_class, set->font [idx]);
+		html_painter_unref_font (manager->painter, set->font [idx]);
 	set->font [idx] = font;
 }
 
@@ -278,21 +250,6 @@ html_font_manager_get_attr (gchar *font_name, gint n)
 	    return g_strdup ("Unknown");
 }
 
-static gchar *
-get_name_from_face (HTMLFontManager *m, const gchar *face)
-{
-	gchar *enc1, *enc2, *rv;
-
-	enc1 = html_font_manager_get_attr (m->variable.face, 13);
-	enc2 = html_font_manager_get_attr (m->variable.face, 14);
-
-	rv = g_strdup_printf ("-*-%s-*-*-*-*-*-*-*-*-*-*-%s-%s", face, enc1, enc2);
-
-	g_free (enc1);
-	g_free (enc2);
-
-	return rv;
-}
 
 static gboolean
 get_points (HTMLFontManager *manager, GtkHTMLFontStyle style)
@@ -301,19 +258,9 @@ get_points (HTMLFontManager *manager, GtkHTMLFontStyle style)
 }
 
 static gpointer
-manager_alloc_font (HTMLFontManager *manager, const gchar *face, GtkHTMLFontStyle style)
+manager_alloc_font (HTMLFontManager *manager, gchar *face, GtkHTMLFontStyle style)
 {
-	gchar *name = manager->variable.face && strcasecmp (face, manager->variable.face)
-		&& manager->fixed.face && strcasecmp (face, manager->fixed.face)
-		? get_name_from_face (manager, face)
-		: g_strdup (face);
-	HTMLFont *font;
-
-	font = html_painter_alloc_font (manager->painter_class, name,
-					get_real_font_size (manager, style), get_points (manager, style), style);
-	g_free (name);
-
-	return font;
+	return html_painter_alloc_font (manager->painter, face, get_real_font_size (manager, style), get_points (manager, style), style);
 }
 
 static gchar *
@@ -381,7 +328,6 @@ html_font_manager_get_font (HTMLFontManager *manager, gchar *face_list, GtkHTMLF
 	HTMLFontSet *set;
 	HTMLFont *font = NULL;
 
-	/* printf ("get font %s (%p)\n", face_list, manager); */
 	font = get_font (manager, &set, face_list, style);
 
 	if (!font) {
@@ -391,7 +337,7 @@ html_font_manager_get_font (HTMLFontManager *manager, gchar *face_list, GtkHTMLF
 			g_assert (set);
 			if (!face_list) {
 				/* default font, so the last chance is to get fixed font */
-				font = html_painter_alloc_font (manager->painter_class, NULL,
+				font = html_painter_alloc_font (manager->painter, NULL,
 								get_real_font_size (manager, style),
 								get_points (manager, style), style);
 				if (!font)
@@ -399,7 +345,7 @@ html_font_manager_get_font (HTMLFontManager *manager, gchar *face_list, GtkHTMLF
 			} else {
 				/* some unavailable non-default font => use default one */
 				font = html_font_manager_get_font (manager, NULL, style);
-				html_font_ref (font, manager->painter_class);
+				html_font_ref (font, manager->painter);
 			}
 			if (font)
 				html_font_set_font (manager, set, style, font);
@@ -410,7 +356,12 @@ html_font_manager_get_font (HTMLFontManager *manager, gchar *face_list, GtkHTMLF
 }
 
 HTMLFont *
-html_font_new (gpointer data, guint space_width, guint nbsp_width, guint tab_width)
+html_font_new (gpointer data, 
+	       guint space_width, 
+	       guint nbsp_width, 
+	       guint tab_width,
+	       guint indent_width,
+	       guint cite_width)
 {
 	HTMLFont *font = g_new (HTMLFont, 1);
 
@@ -418,6 +369,8 @@ html_font_new (gpointer data, guint space_width, guint nbsp_width, guint tab_wid
 	font->space_width = space_width;
 	font->nbsp_width = nbsp_width;
 	font->tab_width = tab_width;
+	font->indent_width = indent_width;
+	font->cite_width = cite_width;
 	font->ref_count = 1;
 
 	return font;
@@ -430,16 +383,16 @@ html_font_destroy (HTMLFont *font)
 }
 
 void
-html_font_ref (HTMLFont *font, HTMLPainterClass *painter_class)
+html_font_ref (HTMLFont *font, HTMLPainter *painter)
 {
-	html_painter_ref_font (painter_class, font);
+	html_painter_ref_font (painter, font);
 	font->ref_count ++;
 }
 
 void
-html_font_unref (HTMLFont *font, HTMLPainterClass *painter_class)
+html_font_unref (HTMLFont *font, HTMLPainter *painter)
 {
-	html_painter_unref_font (painter_class, font);
+	html_painter_unref_font (painter, font);
 	font->ref_count --;
 	if (font->ref_count <= 0)
 		html_font_destroy (font);
