@@ -24,17 +24,20 @@
 #include <string.h>
 #include "gtkhtml.h"
 #include "gtkhtml-private.h"
+#include "gtkhtml-stream.h"
 #include "htmlcolorset.h"
 #include "htmlgdkpainter.h"
 #include "htmlprinter.h"
 #include "htmliframe.h"
 #include "htmlengine-search.h"
+#include "htmlengine-save.h"
 #include "htmlsearch.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
 #include "htmltokenizer.h"
 #include "htmlembedded.h"
 
+#define USE_SCROLLED_WINDOW 1
 #ifndef USE_SCROLLED_WINDOW
 #include <gal/widgets/e-scroll-frame.h>
 #endif
@@ -58,8 +61,7 @@ iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpo
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested",
-				 url, handle);
+	g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
 }
 
 static void
@@ -79,7 +81,7 @@ iframe_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	gboolean ret_val;
 
 	ret_val = FALSE;
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "object_requested", eb, &ret_val);
+	g_signal_emit_by_name (parent, "object_requested", eb, &ret_val);
 	return ret_val;
 }
 
@@ -87,10 +89,10 @@ static void
 iframe_set_gdk_painter (HTMLIFrame *iframe, HTMLPainter *painter)
 {
 	if (painter)
-		gtk_object_ref (GTK_OBJECT (painter));
+		g_object_ref (G_OBJECT (painter));
 	
 	if (iframe->gdk_painter)
-		gtk_object_unref (GTK_OBJECT (iframe->gdk_painter));
+		g_object_unref (G_OBJECT (iframe->gdk_painter));
 
 	iframe->gdk_painter = painter;
 }
@@ -128,7 +130,13 @@ static gint
 calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
 {
-	return html_engine_calc_min_width (GTK_HTML (HTML_IFRAME (o)->html)->engine);
+	HTMLIFrame *iframe;
+
+	iframe = HTML_IFRAME (o);
+	if (iframe->width < 0)
+		return html_engine_calc_min_width (GTK_HTML (HTML_IFRAME (o)->html)->engine);
+	else 
+		return iframe->width;
 }
 
 static void
@@ -170,12 +178,12 @@ draw (HTMLObject *o,
 {
 	HTMLIFrame   *iframe  = HTML_IFRAME (o);
 	HTMLEngine   *e       = GTK_HTML (iframe->html)->engine;
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	if (GTK_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
 		gint pixel_size = html_painter_get_pixel_size (e->painter);
-		html_object_calc_intersection (o, &paint, x, y, width, height);
-		if (art_irect_empty (&paint))
+
+		if (!html_object_intersect (o, &paint, x, y, width, height))
 			return;
 
 		html_object_draw (e->clue, e->painter,
@@ -193,12 +201,12 @@ set_painter (HTMLObject *o, HTMLPainter *painter)
 	HTMLIFrame *iframe;
 
 	iframe = HTML_IFRAME (o);
-	if (GTK_OBJECT_TYPE (GTK_HTML (iframe->html)->engine->painter) != HTML_TYPE_PRINTER) {
+	if (G_OBJECT_TYPE (GTK_HTML (iframe->html)->engine->painter) != HTML_TYPE_PRINTER) {
 		iframe_set_gdk_painter (iframe, GTK_HTML (iframe->html)->engine->painter);
 	}
 	
 	html_engine_set_painter (GTK_HTML (iframe->html)->engine,
-				 GTK_OBJECT_TYPE (painter) != HTML_TYPE_PRINTER ? iframe->gdk_painter : painter);
+				 G_OBJECT_TYPE (painter) != HTML_TYPE_PRINTER ? iframe->gdk_painter : painter);
 }
 
 static void
@@ -217,7 +225,19 @@ forall (HTMLObject *self,
 static gint
 check_page_split (HTMLObject *self, gint y)
 {
-	return html_object_check_page_split (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue, y);
+	gint y1, y2;
+	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
+
+	y1 = self->y - self->ascent + e->topBorder;
+	y2 = self->y + self->descent + e->topBorder;
+	
+	if (y1 > y) 
+		return 0;
+
+	if (y >= y1 && y < y2)
+		return html_object_check_page_split (e->clue, y - y1) + y1;
+	
+	return y;
 }
 
 static void
@@ -229,18 +249,17 @@ copy (HTMLObject *self,
 
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 	/*
-	html_iframe_init (s, &html_iframe_class,
+	html_iframe_init (d, &html_iframe_class,
 			  HTML_EMBEDDED (dest)->parent,
 			  s->url,
 			  s->width,
 			  s->height,
 			  s->frameborder);
 	*/
-	d->scroll = NULL;
+	d->scroll = s->scroll;
 	d->html = gtk_html_new ();
+
 	d->gdk_painter = NULL;
-	d->old_painter = NULL;
-	d->parent_painter = NULL;
 
 	d->url = g_strdup (s->url);
 	d->width = s->width;
@@ -255,7 +274,7 @@ op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList
 	GtkHTML *html;
 
 	dup = html_object_dup (self);
-	html = HTML_IFRAME (dup)->html;
+	html = GTK_HTML (HTML_IFRAME (dup)->html);
 	clue = GTK_HTML (HTML_IFRAME (self)->html)->engine->clue;
 	GTK_HTML (HTML_IFRAME (dup)->html)->engine->clue =
 		html_object_op_copy (clue, dup, GTK_HTML (HTML_IFRAME (self)->html)->engine,
@@ -266,10 +285,32 @@ op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList
 	return dup;
 }
 
+void
+html_iframe_set_margin_width (HTMLIFrame *iframe, gint margin_width)
+{
+	HTMLEngine *e;
+
+	e = GTK_HTML (iframe->html)->engine;
+
+	e->leftBorder = e->rightBorder = margin_width;
+	html_engine_schedule_redraw (e);
+}
+
+void
+html_iframe_set_margin_height (HTMLIFrame *iframe, gint margin_height)
+{
+	HTMLEngine *e;
+
+	e = GTK_HTML (iframe->html)->engine;
+
+	e->bottomBorder = e->topBorder = margin_height;
+	html_engine_schedule_redraw (e);
+}
+
 void
 html_iframe_set_scrolling (HTMLIFrame *iframe, GtkPolicyType scroll)
 {
-#if E_USE_SCROLLED_WINDOW
+#if USE_SCROLLED_WINDOW
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (iframe->scroll),
 					scroll, scroll);
 #else
@@ -292,6 +333,9 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 
 	iframe = HTML_IFRAME (o);
 	e      = GTK_HTML (iframe->html)->engine;
+	
+	if (HTML_EMBEDDED (o)->widget == NULL)
+		return TRUE;
 
 	if ((iframe->width < 0) && (iframe->height < 0)) {
 		e->width = o->max_width;
@@ -300,7 +344,7 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 		height = html_engine_get_doc_height (e);
 		width = html_engine_get_doc_width (e);
 
-		gtk_widget_set_usize (iframe->scroll, width, height);
+		gtk_widget_set_size_request (iframe->scroll, width, height);
 		gtk_widget_queue_resize (iframe->scroll);
 		
 		html_iframe_set_scrolling (iframe, GTK_POLICY_NEVER);
@@ -378,18 +422,10 @@ check_point (HTMLObject *self,
 	    || y >= self->y + self->descent || y < self->y - self->ascent)
 		return NULL;
 
-	x -= self->x + e->leftBorder - e->x_offset;
-	y -= self->y - self->ascent + e->topBorder - e->y_offset;
+	x -= self->x - e->x_offset;
+	y -= self->y - self->ascent - e->y_offset;
 
-	if (for_cursor && (x < 0 || y < e->clue->y - e->clue->ascent))
-		return html_object_check_point (e->clue, e->painter, 0, e->clue->y - e->clue->ascent,
-						offset_return, for_cursor);
-
-	if (for_cursor && (x > e->clue->width - 1 || y > e->clue->y + e->clue->descent - 1))
-		return html_object_check_point (e->clue, e->painter, e->clue->width - 1, e->clue->y + e->clue->descent - 1,
-						offset_return, for_cursor);
-
-	return html_object_check_point (e->clue, e->painter, x, y, offset_return, for_cursor);
+	return html_engine_get_object_at (e, x, y, offset_return, for_cursor);
 }
 
 static gboolean
@@ -428,6 +464,86 @@ select_range (HTMLObject *self,
 					 start, length, queue_draw);
 } */
 
+static gboolean
+save (HTMLObject *s,
+      HTMLEngineSaveState *state)
+{
+	HTMLIFrame *iframe = HTML_IFRAME (s);
+	HTMLEngineSaveState *buffer;
+	HTMLEngine *e;
+
+	e = GTK_HTML (iframe->html)->engine;
+	
+	/*
+	 * FIXME: we should actually save the iframe definition if inline_frames is not
+	 * set, but that is a feature and not critical for release.  We should also probably
+	 * wrap the body in a <table> tag with a bg etc.
+	 */
+	if (state->inline_frames && e->clue) {
+		buffer = html_engine_save_buffer_new (e, state->inline_frames);
+		html_object_save (e->clue, buffer);
+		if (state->error || 
+		    !html_engine_save_output_string (state, html_engine_save_buffer_peek_text (buffer))) {
+			html_engine_save_buffer_free (buffer);
+			return FALSE;
+		}
+		html_engine_save_buffer_free (buffer);
+	} else {
+		HTMLEngine *e = GTK_HTML (iframe->html)->engine;
+
+		if (!html_engine_save_output_string (state, "<IFRAME SRC=\"%s\"", iframe->url))
+			 return FALSE;
+        
+		 if (iframe->width >= 0)
+			 if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", iframe->width))
+				 return FALSE;
+
+		 if (iframe->width >= 0)
+			 if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", iframe->width))
+				 return FALSE;
+
+		 if (e->topBorder != TOP_BORDER || e->bottomBorder != BOTTOM_BORDER)
+			 if (!html_engine_save_output_string (state, " MARGINHEIGHT=\"%d\"", e->topBorder))
+				 return FALSE;
+
+		 if (e->leftBorder != LEFT_BORDER || e->rightBorder != RIGHT_BORDER)
+			 if (!html_engine_save_output_string (state, " MARGINWIDTH=\"%d\"", e->leftBorder))
+				 return FALSE;
+
+		 if (!html_engine_save_output_string (state, " FRAMEBORDER=\"%d\"", iframe->frameborder))
+			 return FALSE;
+		 
+		 if (!html_engine_save_output_string (state, "></IFRAME>"))
+		     return FALSE;
+	}
+	return TRUE;
+}
+
+static gboolean
+save_plain (HTMLObject *s,
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
+{
+	HTMLIFrame *iframe = HTML_IFRAME (s);
+	HTMLEngineSaveState *buffer;
+	HTMLEngine *e;
+
+	e = GTK_HTML (iframe->html)->engine;
+
+	if (state->inline_frames && e->clue) {
+		buffer = html_engine_save_buffer_new (e, state->inline_frames);
+		html_object_save_plain (e->clue, buffer, requested_width);
+		if (state->error || 
+		    !html_engine_save_output_string (state, html_engine_save_buffer_peek_text (buffer))) {
+			html_engine_save_buffer_free (buffer);
+			return FALSE;
+		}
+		html_engine_save_buffer_free (buffer);
+	}
+
+	return TRUE;
+}
+
 static void
 destroy (HTMLObject *o)
 {
@@ -438,13 +554,7 @@ destroy (HTMLObject *o)
 	g_free (iframe->url);
 
 	if (iframe->html) {
-		if (iframe->old_painter)
-			gtk_object_unref (GTK_OBJECT (iframe->old_painter));
-		if (iframe->parent_painter)
-			gtk_object_unref (GTK_OBJECT (iframe->parent_painter));
-		iframe->old_painter = iframe->parent_painter = NULL;
-
-		gtk_signal_disconnect_by_data (GTK_OBJECT (iframe->html), o);
+		g_signal_handlers_disconnect_matched (iframe->html, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, o);
 		iframe->html = NULL;
 	}
 
@@ -467,6 +577,7 @@ html_iframe_init (HTMLIFrame *iframe,
 	GtkHTML   *parent_html;
 	GtkHTMLStream *handle;
 	GtkWidget *scrolled_window;
+	gint depth;
 
 	g_assert (GTK_IS_HTML (parent));
 	parent_html = GTK_HTML (parent);
@@ -482,6 +593,11 @@ html_iframe_init (HTMLIFrame *iframe,
 					border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
 
 #endif
+	/*
+	 * FIXME
+	 * are we missing:
+	 * gtk_widget_show (scrolled_window); here?
+	 */
 
 	iframe->scroll = scrolled_window;
 	html_iframe_set_scrolling (iframe, GTK_POLICY_AUTOMATIC);
@@ -489,22 +605,16 @@ html_iframe_init (HTMLIFrame *iframe,
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
 
-	/* use parent_html font_manager */
-	gtk_object_ref (GTK_OBJECT (new_html->engine->painter));
-	gtk_object_ref (GTK_OBJECT (parent_html->engine->painter));
-	iframe->old_painter = new_html->engine->painter;
-	iframe->parent_painter = parent_html->engine->painter;
-
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
 	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
-	gtk_object_unref (GTK_OBJECT (new_tokenizer));
+	g_object_unref (G_OBJECT (new_tokenizer));
 	new_tokenizer = NULL;
 
 	gtk_html_set_default_content_type (new_html,
 					   parent_html->priv->content_type);
 	iframe->html = new_widget;
-	gtk_html_set_iframe_parent (new_html, parent, HTML_OBJECT (iframe));
+	depth = gtk_html_set_iframe_parent (new_html, parent, HTML_OBJECT (iframe));
 	gtk_container_add (GTK_CONTAINER (scrolled_window), new_widget);
 	gtk_widget_show (new_widget);
 
@@ -512,15 +622,14 @@ html_iframe_init (HTMLIFrame *iframe,
 	iframe->width = width;
 	iframe->height = height;
 	iframe->gdk_painter = NULL;
+	iframe->frameborder = border;
 	gtk_html_set_base (new_html, src);
 
 	handle = gtk_html_begin (new_html);
 
 	new_html->engine->clue->parent = HTML_OBJECT (iframe);
-
-	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
-			    GTK_SIGNAL_FUNC (iframe_url_requested),
-			    (gpointer)iframe);
+	
+	g_signal_connect (new_html, "url_requested", G_CALLBACK (iframe_url_requested), iframe);
 #if 0
 	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
 	 * on_url and link_clicked are emitted from the toplevel widget not
@@ -533,32 +642,24 @@ html_iframe_init (HTMLIFrame *iframe,
 			    GTK_SIGNAL_FUNC (iframe_link_clicked),
 			    (gpointer)iframe);	
 #endif 
-	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
-			    GTK_SIGNAL_FUNC (iframe_size_changed),
-			    (gpointer)iframe);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
-			    GTK_SIGNAL_FUNC (iframe_set_base),
-			    (gpointer)iframe);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "object_requested",
-			    GTK_SIGNAL_FUNC (iframe_object_requested),
-			    (gpointer)iframe);	
+	g_signal_connect (new_html, "size_changed", G_CALLBACK (iframe_size_changed), iframe);	
+	g_signal_connect (new_html, "set_base", G_CALLBACK (iframe_set_base), iframe);	
+	g_signal_connect (new_html, "object_requested", G_CALLBACK (iframe_object_requested), iframe);	
 
 	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent_html->engine), 
-				 "url_requested", src, handle);
-
-	gtk_widget_set_usize (scrolled_window, width, height);
+	if (depth < 10)
+		g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
+	
+	gtk_widget_set_size_request (scrolled_window, width, height);
 
 	gtk_widget_show (scrolled_window);	
 
 	html_embedded_set_widget (em, scrolled_window);
 
-	gtk_signal_connect(GTK_OBJECT (scrolled_window), "button_press_event",
-			   GTK_SIGNAL_FUNC (html_iframe_grab_cursor), NULL);
+	g_signal_connect (scrolled_window, "button_press_event", G_CALLBACK (html_iframe_grab_cursor), NULL);
 
 	/* inherit the current colors from our parent */
 	html_colorset_set_unchanged (new_html->engine->defaultSettings->color_set,
@@ -601,6 +702,8 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	parent_class = &html_embedded_class;
 
 	object_class->destroy                 = destroy;
+	object_class->save                    = save;
+	object_class->save_plain              = save_plain;
 	object_class->calc_size               = calc_size;
 	object_class->calc_min_width          = calc_min_width;
 	object_class->set_painter             = set_painter;
