@@ -26,13 +26,16 @@
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
-#include <gal/widgets/e-unicode.h>
+
+#include <pango/pango.h>
 
 #include "htmltext.h"
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
+#include "htmlgdkpainter.h"
+#include "htmlplainpainter.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-cut-and-paste.h"
@@ -53,6 +56,7 @@ static void         spell_error_destroy     (SpellError *se);
 static void         move_spell_errors       (GList *spell_errors, guint offset, gint delta);
 static GList *      remove_spell_errors     (GList *spell_errors, guint offset, guint len);
 static guint        get_words               (const gchar *s);
+static GList *      get_glyphs              (HTMLText *text, HTMLPainter *painter);
 static void         remove_text_slaves      (HTMLObject *self);
 
 /* static void
@@ -62,17 +66,6 @@ debug_spell_errors (GList *se)
 		printf ("SE: %4d, %4d\n", ((SpellError *) se->data)->off, ((SpellError *) se->data)->len);
 } */
 
-#ifndef GAL_NOT_SLOW
-/* NOTE see htmltextslave.c for an explanation and the h_utf8 decls */
-gint h_utf8_pointer_to_offset (const gchar *str, const gchar *pos);
-gint h_utf8_strlen (const gchar *p, gint max);
-gchar *h_utf8_offset_to_pointer  (const gchar *str, gint         offset);
-
-#define g_utf8_strlen h_utf8_strlen
-#define g_utf8_offset_to_pointer h_utf8_offset_to_pointer
-#define g_utf8_pointer_to_offset h_utf8_pointer_to_offset
-#endif
-
 static inline gboolean
 is_in_the_save_cluev (HTMLObject *text, HTMLObject *o)
 {
@@ -123,16 +116,16 @@ get_tags (const HTMLText *text,
 		|| !html_color_equal (text->color, pt->color)
 		|| (pt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK))) {
 		if (!std_color) {
-			g_string_sprintfa (ot, "<FONT COLOR=\"#%02x%02x%02x\"",
-					   text->color->color.red   >> 8,
-					   text->color->color.green >> 8,
-					   text->color->color.blue  >> 8);
+			g_string_append_printf (ot, "<FONT COLOR=\"#%02x%02x%02x\"",
+						text->color->color.red   >> 8,
+						text->color->color.green >> 8,
+						text->color->color.blue  >> 8);
 			font_tag = TRUE;
 		}
 		if (!std_size) {
 			if (!font_tag)
 				g_string_append (ot, "<FONT");
-			g_string_sprintfa (ot, " SIZE=\"%d\"", font_style & GTK_HTML_FONT_STYLE_SIZE_MASK);
+			g_string_append_printf (ot, " SIZE=\"%d\"", font_style & GTK_HTML_FONT_STYLE_SIZE_MASK);
 		}
 		g_string_append_c (ot, '>');
 	}
@@ -141,8 +134,8 @@ get_tags (const HTMLText *text,
 	    && (!nt
 		|| !html_color_equal (text->color, nt->color)
 		|| (nt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK))) {
-		g_string_append (ct, "</FONT>");
-	}
+  		g_string_append (ct, "</FONT>");
+  	}
 
 	/* bold tag */
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
@@ -193,6 +186,29 @@ get_tags (const HTMLText *text,
 
 /* HTMLObject methods.  */
 
+inline static void
+glyphs_destroy (GList *glyphs)
+{
+	GList *l;
+
+	for (l = glyphs; l; l = l->next)
+		pango_glyph_string_free ((PangoGlyphString *) l->data);
+	g_list_free (glyphs);
+}
+
+inline static void
+items_destroy (HTMLText *text)
+{
+	if (text->items) {
+		GList *l;
+
+		for (l = text->items; l; l = l->next)
+			pango_item_free ((PangoItem *) l->data);
+		g_list_free (text->items);
+		text->items = NULL;
+	}
+}
+
 static void
 copy (HTMLObject *s,
       HTMLObject *d)
@@ -223,6 +239,7 @@ copy (HTMLObject *s,
 
 	dest->words      = 0;
 	dest->word_width = NULL;
+	dest->items = NULL;
 }
 
 /* static void
@@ -328,6 +345,7 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 		text->spell_errors = remove_spell_errors (text->spell_errors, begin, end - begin);
 		move_spell_errors (text->spell_errors, end, - (end - begin));
 		html_text_convert_nbsp (text, TRUE);
+		items_destroy (text);
 	} else {
 		text->spell_errors = remove_spell_errors (text->spell_errors, 0, text->text_len);
 		html_object_move_cursor_before_remove (HTML_OBJECT (text), e);
@@ -399,6 +417,8 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, G
 	g_free (to_free);
 	html_text_convert_nbsp (t1, TRUE);
 	html_object_change_set (self, HTML_CHANGE_ALL_CALC);
+	items_destroy (t1);
+	items_destroy (t2);
 
 	html_text_clear_word_width (t1);
 	/* html_text_request_word_width (t1, e->painter); */
@@ -419,7 +439,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 
 	g_assert (self->parent);
 
-	html_clueflow_remove_text_slaves (HTML_CLUEFLOW (self->parent));
+	html_clue_remove_text_slaves (HTML_CLUE (self->parent));
 
 	t1              = HTML_TEXT (self);
 	dup             = html_object_dup (self);
@@ -470,6 +490,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 
 	html_text_clear_word_width (HTML_TEXT (self));
 	html_text_clear_word_width (HTML_TEXT (dup));
+	items_destroy (HTML_TEXT (self));
 
 	level--;
 	if (level)
@@ -479,48 +500,74 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 static gboolean
 calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
-	HTMLText *text = HTML_TEXT (self);
-	GtkHTMLFontStyle style = html_text_get_font_style (text);
+	/* RM2 HTMLText *text = HTML_TEXT (self);
+	   GtkHTMLFontStyle style = html_text_get_font_style (text); */
 
 	self->width = 0;
-	self->ascent = html_painter_calc_ascent (painter, style, text->face);
-	self->descent = html_painter_calc_descent (painter, style, text->face);
+	/* self->ascent = 0; FIX2? */
+	/* self->descent = 0; FIX2? */
 
 	return FALSE;
 }
 
+const gchar *
+html_utf8_strnchr (const gchar *s, gchar c, gint len, gint *offset)
+{
+	const gchar *res = NULL;
+
+	*offset = 0;
+	while (s && *s && *offset < len) {
+		if (*s == c) {
+			res = s;
+			break;
+		}
+		s = g_utf8_next_char (s);
+		(*offset) ++;
+	}
+
+	return res;
+}
+
 gint
-html_text_text_line_length (const gchar *text, gint line_offset, guint len)
+html_text_text_line_length (const gchar *text, gint *line_offset, guint len, gint *tabs)
 {
 	const gchar *tab, *found_tab;
-	gint lo, cl, l, skip, sum_skip;
+	gint cl, l, skip, sum_skip;
 
-	/* printf ("lo: %d o: %d t: '%s'\n", line_offset, len, text); */
+	/* printf ("lo: %d len: %d t: '%s'\n", line_offset, len, text); */
+	if (tabs)
+		*tabs = 0;
 	l = 0;
-	lo = line_offset;
-	sum_skip = 0;
+	sum_skip = skip = 0;
 	tab = text;
-	while (tab && (found_tab = strchr (tab, '\t')) && l < len) {
-		cl   = g_utf8_pointer_to_offset (tab, found_tab);
+	while (tab && (found_tab = html_utf8_strnchr (tab, '\t', len - l, &cl)) && l < len) {
 		l   += cl;
 		if (l >= len)
 			break;
-		lo  += cl;
-		skip = 8 - (lo % 8);
+		if (*line_offset != -1) {
+			*line_offset  += cl;
+			skip = 8 - (*line_offset % 8);
+		}
 		tab  = found_tab + 1;
-		lo  += skip;
-		sum_skip += skip - 1;
+		*line_offset  += skip;
+		if (*line_offset != -1)
+			sum_skip += skip - 1;
 		l ++;
+		if (tabs)
+			(*tabs) ++;
 	}
 
+	(*line_offset) += len - l;
+	/* printf ("ll: %d\n", len + sum_skip); */
+
 	return len + sum_skip;
 }
 
 static guint
 get_line_length (HTMLObject *self, HTMLPainter *p, gint line_offset)
 {
-	return html_clueflow_tabs (HTML_CLUEFLOW (self->parent), p) || line_offset == -1
-		? html_text_text_line_length (HTML_TEXT (self)->text, line_offset, HTML_TEXT (self)->text_len)
+	return html_clueflow_tabs (HTML_CLUEFLOW (self->parent), p)
+		? html_text_text_line_length (HTML_TEXT (self)->text, &line_offset, HTML_TEXT (self)->text_len, NULL)
 		: HTML_TEXT (self)->text_len;
 }
 
@@ -546,32 +593,105 @@ get_words (const gchar *s)
 	return words;
 }
 
+static gint
+word_size (gint cl, gint so, gint eo, GList **items, GList **glyphs, gint *width, gint *asc, gint *dsc)
+{
+	PangoItem *item;
+	PangoRectangle rect;
+	gint ceo;
+
+	*width = *asc = *dsc = 0;
+	while (so < eo) {
+		item = (PangoItem *) (*items)->data;
+		ceo = MIN (cl + item->num_chars, eo);
+		pango_glyph_string_extents_range ((PangoGlyphString *)(*glyphs)->data, so - cl, ceo - cl, item->analysis.font, NULL, &rect);
+
+		*width += PANGO_PIXELS (rect.width);
+		*asc = MAX (PANGO_PIXELS (PANGO_ASCENT (rect)), *asc);
+		*dsc = MAX (PANGO_PIXELS (PANGO_DESCENT (rect)), *dsc);
+
+		if (cl + item->num_chars <= eo) {
+			cl += item->num_chars;
+			*items = (*items)->next;
+			*glyphs = (*glyphs)->next;
+		}
+		so += ceo - so;
+	}
+
+	/* printf ("word width: %d\n", *width); */
+
+	return cl;
+}
+
 static void
 calc_word_width (HTMLText *text, HTMLPainter *painter, gint line_offset)
 {
 	GtkHTMLFontStyle style;
+	GList *gl, *glyphs = NULL;
+	GList *il, *items = NULL;
 	HTMLFont *font;
+	HTMLObject *obj = HTML_OBJECT (text);
 	gchar *begin, *end;
-	gint i;
+	gint i, cl, width, asc, dsc, start_offset, end_offset;
+
+	/* printf ("calc ww begin\n"); */
 
 	text->words      = get_words (text->text);
 	if (text->word_width)
 		g_free (text->word_width);
 	text->word_width = g_new (guint, text->words);
 	style = html_text_get_font_style (text);
-	font = html_painter_get_html_font (painter, text->face, style);
+	font = html_font_manager_get_font (&painter->font_manager, text->face, style);
 
-	begin            = text->text;
+	obj->ascent = obj->descent = 0;
+
+	if (text->text_len) {
+		items = html_text_get_items (text, painter);
+		if (items)
+			glyphs = get_glyphs (text, painter);
+	}
+
+	/* printf ("calc ww m1\n"); */
+	begin = text->text;
+	start_offset = end_offset = 0;
+	il = items;
+	gl = glyphs;
+	cl = 0;
 	for (i = 0; i < text->words; i++) {
 		end   = strchr (begin + (i ? 1 : 0), ' ');
-		text->word_width [i] = (i ? text->word_width [i - 1] : 0)
-			+ html_painter_calc_text_width_bytes (painter,
-							      begin, end ? end - begin : strlen (begin),
-							      &line_offset, font, style);
+
+		if (il && gl) {
+			/* end_offset = start_offset + (end ? g_utf8_pointer_to_offset (begin, end) : g_utf8_strlen (begin, -1)); */
+			end_offset = start_offset + (end ? g_utf8_pointer_to_offset (begin, end) : g_utf8_strlen (begin, -1));
+			/* printf ("start offset: %d (%d)\n", start_offset, end_offset - start_offset); */
+			cl = word_size (cl, start_offset, end_offset, &il, &gl, &width, &asc, &dsc);
+		} else
+			html_painter_calc_text_size_bytes (painter,
+							   begin, end ? end - begin : strlen (begin), NULL, NULL, 0,
+							   NULL, font, style, &width, &asc, &dsc);
+		text->word_width [i] = (i ? text->word_width [i - 1] : 0) + width;
+
+		if (obj->ascent < asc)
+			obj->ascent = asc;
+		if (obj->descent < dsc)
+			obj->descent = dsc;
 		begin = end;
+		start_offset = end_offset;
+	}
+	/* printf ("calc ww m2\n"); */
+
+	if (glyphs) {
+		glyphs_destroy (glyphs);
+		glyphs = NULL;
+	}
+	if (text->text_len == 0) {
+		/* FIXME: cache items and glyphs? */
+		html_painter_calc_text_size_bytes (painter, " ", 1, NULL, NULL, 0, NULL, font, style, &width, &obj->ascent, &obj->descent);
 	}
 
 	HTML_OBJECT (text)->change &= ~HTML_CHANGE_WORD_WIDTH;
+
+	/* printf ("calc ww end\n"); */
 }
 
 void
@@ -580,6 +700,9 @@ html_text_request_word_width (HTMLText *text, HTMLPainter *painter)
 	if (!text->word_width || (HTML_OBJECT (text)->change & HTML_CHANGE_WORD_WIDTH)) {
 		gint offset;
 
+		if (HTML_OBJECT (text)->change & HTML_CHANGE_WORD_WIDTH)
+			items_destroy (text);
+
 		offset = html_text_get_line_offset (text, painter);
 		calc_word_width (text, painter, offset);
 	}
@@ -627,9 +750,6 @@ ht_fit_line (HTMLObject *o,
 
 	text = HTML_TEXT (o);
 
-	if (o->flags & HTML_OBJECT_FLAG_NEWLINE)
-		return HTML_FIT_COMPLETE;
-
 	remove_text_slaves (o);
 
 	/* Turn all text over to our slaves */
@@ -650,8 +770,8 @@ forward_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 
 	/* find prev/next object */
 	obj = begin
-		? html_object_prev_not_slave (HTML_OBJECT (text))
-		: html_object_next_not_slave (HTML_OBJECT (text));
+		? html_object_next_not_slave (HTML_OBJECT (text))
+		: html_object_prev_not_slave (HTML_OBJECT (text));
 
 	/* if not found or not text return 0, otherwise forward get_nb_with there */
 	if (!obj || !html_object_is_text (obj))
@@ -681,6 +801,40 @@ get_next_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		return html_text_get_nb_width (HTML_TEXT (obj), painter, begin);
 }
 
+static inline gint
+word_tabs (HTMLText *text, HTMLPainter *p, gint idx)
+{
+	gchar *str, *end;
+	gint rv = 0, line_offset, t1, t2, l1, l2;
+
+	if (!html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (text)->parent), p))
+		return 0;
+
+	str = text->text;
+	while (idx > 0 && str && *str) {
+		str = strchr (str, ' ');
+		if (str)
+			str ++;
+		idx --;
+	}
+
+	if (!str || !*str)
+		return 0;
+
+	end = strchr (str, ' ');
+	if (!end)
+		end = str + strlen (str);
+
+	line_offset = html_text_get_line_offset (text, p);
+	l1 = html_text_text_line_length (text->text, &line_offset, g_utf8_pointer_to_offset (text->text, str), &t1);
+	l2 = html_text_text_line_length (text->text, &line_offset, g_utf8_pointer_to_offset (str, end), &t2);
+
+	rv = l2 - l1 - g_utf8_pointer_to_offset (str, end);
+
+	/* printf ("tabs delta %d\n", rv); */
+	return rv;
+}
+
 static guint
 word_width (HTMLText *text, HTMLPainter *p, guint i)
 {
@@ -688,7 +842,8 @@ word_width (HTMLText *text, HTMLPainter *p, guint i)
 
 	return text->word_width [i]
 		- (i > 0 ? text->word_width [i - 1]
-		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0);
+		   + html_painter_get_space_width (p, html_text_get_font_style (text), text->face) : 0)
+		+ word_tabs (text, p, i)*html_painter_get_space_width (p, html_text_get_font_style (text), text->face);
 }
 
 /* return non-breakable text width on begin/end of this text */
@@ -710,27 +865,102 @@ html_text_get_nb_width (HTMLText *text, HTMLPainter *painter, gboolean begin)
 		+ (text->words == 1 ? get_next_nb_width (text, painter, begin) : 0);
 }
 
+static GList *
+get_glyphs (HTMLText *text, HTMLPainter *painter)
+{
+	GList *glyphs = NULL, *items = html_text_get_items (text, painter);
+
+	if (items) {
+		PangoGlyphString *str = NULL;
+		PangoItem *item;
+		GList *il;
+		gchar *heap = NULL, *translated;
+		gint bytes;
+
+		bytes = strlen (text->text);
+		if (bytes > HTML_ALLOCA_MAX)
+			heap = translated = g_malloc (bytes);
+		else 
+			translated = alloca (bytes);
+		html_replace_tabs (text->text, translated, bytes);
+		for (il = items; il; il = il->next) {
+			item = (PangoItem *) il->data;
+			if (item->length) {
+				str = pango_glyph_string_new ();
+				pango_shape (translated + item->offset, item->length, &item->analysis, str);
+				glyphs = g_list_prepend (glyphs, str);
+			}
+		}
+		glyphs = g_list_reverse (glyphs);
+		g_free (heap);
+	}
+
+	return glyphs;
+}
+
+GList *
+html_text_get_items (HTMLText *text, HTMLPainter *painter)
+{
+	if (!HTML_IS_GDK_PAINTER (painter) && !HTML_IS_PLAIN_PAINTER (painter))
+		return NULL;
+	if (!text->items) {
+		PangoContext *pc = HTML_GDK_PAINTER (painter)->pc;
+		PangoAttrList *attrs;
+		gchar *translated, *heap = NULL;
+		guint bytes;
+
+		bytes = strlen (text->text);
+		if (bytes > HTML_ALLOCA_MAX)
+			heap = translated = g_malloc (bytes);
+		else 
+			translated = alloca (bytes);
+
+		html_replace_tabs (text->text, translated, bytes);
+		pango_context_set_font_description (pc, html_painter_get_font (painter, text->face, html_text_get_font_style (text)));
+		attrs = pango_attr_list_new ();
+		text->items = pango_itemize (pc, translated, 0, bytes, attrs, NULL);
+		pango_attr_list_unref (attrs);
+
+		g_free (heap);
+	}
+	return text->items;
+}
+
 static gint
 calc_min_width (HTMLObject *self, HTMLPainter *painter)
 {
 	HTMLText *text = HTML_TEXT (self);
-	HTMLObject *obj;
+	HTMLObject *prev, *next, *obj;
 	guint i, w, mw;
 
+	items_destroy (text);
+	html_text_get_items (text, painter);
 	html_text_request_word_width (text, painter);
 	mw = 0;
 
 	for (i = 0; i < text->words; i++) {
 		w = word_width (text, painter, i);
+		prev = next = NULL;
 		if (i == 0) {
 			obj = html_object_prev_not_slave (self);
-			if (obj && html_object_is_text (obj))
+			if (obj && html_object_is_text (obj)) {
 				w += html_text_get_nb_width (HTML_TEXT (obj), painter, FALSE);
-		} else if (i == text->words - 1) {
-			obj = html_object_next_not_slave (self);
-			if (obj && html_object_is_text (obj))
-				w += html_text_get_nb_width (HTML_TEXT (obj), painter, TRUE);
+				prev = obj;
+			}
 		}
+		if (i == text->words - 1) {
+			obj = html_object_next_not_slave (self);
+			if (obj && html_object_is_text (obj)) {
+				w += html_text_get_nb_width (HTML_TEXT (obj), painter, TRUE);
+				next = obj;
+			}
+		}
+
+		if (prev && prev->min_width < w)
+			prev->min_width = w;
+		if (next && next->min_width < w)
+			next->min_width = w;
+
 		if (w > mw)
 			mw = w;
 	}
@@ -844,7 +1074,7 @@ is_convert_nbsp_needed (const gchar *s, gint *delta_out)
 
 	op = p = s;
 	white_space = 0;
-	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
+	while (*p && (uc = g_utf8_get_char (p)) && (p = g_utf8_next_char (p))) {
 		if (uc == ENTITY_NBSP || uc == ' ') {
 			rv = check_prev_white (rv, white_space, last_white, delta_out);
 			white_space ++;
@@ -896,7 +1126,7 @@ convert_nbsp (gchar *fill, const gchar *p)
 	op = p;
 	white_space = 0;
 
-	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
+	while (*p && (uc = g_utf8_get_char (p)) && (p = g_utf8_next_char (p))) {
 		if (uc == ENTITY_NBSP || uc == ' ') {
 			write_prev_white_space (white_space, &fill);
 			white_space ++;
@@ -1102,6 +1332,7 @@ destroy (HTMLObject *obj)
 	g_free (text->text);
 	g_free (text->word_width);
 	g_free (text->face);
+	items_destroy (text);
 
 	HTML_OBJECT_CLASS (parent_class)->destroy (obj);
 }
@@ -1207,9 +1438,6 @@ append_selection_string (HTMLObject *self,
 	if (text->select_length == 0)
 		return;
 
-	/* FIXME: we need a `g_string_append()' that takes the number of
-           characters to append as an extra parameter.  */
-
 	p    = html_text_get_text (text, text->select_start);
 	last = g_utf8_offset_to_pointer (p, text->select_length);
 	
@@ -1217,10 +1445,8 @@ append_selection_string (HTMLObject *self,
 	last = html_text_get_text (text,
 				   text->select_start + text->select_length);
 	*/
-	for (; p < last;) {
-		g_string_append_c (buffer, *p);
-		p++;
-	}
+	html_engine_save_string_append_nonbsp (buffer, p, last - p);
+
 }
 
 static void
@@ -1284,19 +1510,21 @@ get_cursor_base (HTMLObject *self,
 			if (offset > slave->posStart) {
 				HTMLText *text;
 				GtkHTMLFontStyle font_style;
-				gint line_offset;
+				gint line_offset, width, asc, dsc;
+				gchar *slave_text;
 
 				text = HTML_TEXT (self);
 
 				font_style = html_text_get_font_style (text);
-				line_offset = html_text_slave_get_line_offset (slave,
-									       html_text_get_line_offset (HTML_TEXT (self),
-													  painter),
-									       slave->posStart, painter);
-				*x += html_painter_calc_text_width (painter,
-								    html_text_get_text (text, slave->posStart),
-								    offset - slave->posStart, &line_offset,
-								    font_style, text->face);
+				line_offset = html_text_slave_get_line_offset (slave, 0, painter);
+				/* FIXME: cache items and glyphs? */
+				slave_text = html_text_get_text (text, slave->posStart);
+				html_painter_calc_text_size (painter,
+							     slave_text,
+							     offset - slave->posStart, NULL, NULL, 0, &line_offset,
+							     font_style, text->face, &width, &asc, &dsc);
+
+				*x += width;
 			}
 
 			return;
@@ -1360,13 +1588,14 @@ html_text_class_init (HTMLTextClass *klass,
 }
 
 static gint
-text_len (const gchar *str, gint len)
+text_len (const gchar **str, gint len)
 {
-	if (len == -1) {
-		return g_utf8_validate (str, -1, NULL) ? g_utf8_strlen (str, -1) : 0;
+	if (g_utf8_validate (*str, -1, NULL))
+		return len != -1 ? len : g_utf8_strlen (*str, -1);
+	else {
+		*str = "[?]";
+		return 3;
 	}
-
-	return len;
 }
 
 void
@@ -1381,8 +1610,8 @@ html_text_init (HTMLText *text,
 
 	html_object_init (HTML_OBJECT (text), HTML_OBJECT_CLASS (klass));
 
-	text->text          = len == -1 ? g_strdup (str) : g_strndup (str, g_utf8_offset_to_pointer (str, len) - str);
-	text->text_len      = text_len (str, len);
+	text->text_len      = text_len (&str, len);
+	text->text          = g_strndup (str, g_utf8_offset_to_pointer (str, text->text_len) - str);
 	text->font_style    = font_style;
 	text->face          = NULL;
 	text->color         = color;
@@ -1391,6 +1620,7 @@ html_text_init (HTMLText *text,
 	text->select_length = 0;
 	text->word_width    = NULL;
 	text->words         = 0;
+	text->items         = NULL;
 
 	html_color_ref (color);
 }
@@ -1479,8 +1709,8 @@ void
 html_text_set_text (HTMLText *text, const gchar *new_text)
 {
 	g_free (text->text);
+	text->text_len = text_len (&new_text, -1);
 	text->text = g_strdup (new_text);
-	text->text_len = text_len (new_text, -1);
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 }
 
@@ -1674,10 +1904,10 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 {
 	regmatch_t pmatch [2];
 	gint i;
-	gboolean rv = FALSE;
+	gboolean rv = FALSE, exec = TRUE;
 	gint saved_position;
 	gunichar uc;
-	char *str;
+	char *str, *cur;
 
 	if (!offset)
 		return FALSE;
@@ -1688,11 +1918,23 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 	html_undo_level_begin (engine->undo, "Magic link", "Remove magic link");
 	saved_position = engine->cursor->position;
 
-	str = html_text_get_text (text, offset);
+	cur = str = html_text_get_text (text, offset);
+
+	/* check forward to ensure chars are < 0x80, could be removed once we have utf8 regex */
+	do {
+		cur = g_utf8_next_char (cur);
+		if (cur && *cur && (uc = g_utf8_get_char (cur)) >= 0x80)
+			exec = FALSE;
+	} while (exec && cur && *cur && uc != ' ' && uc != ENTITY_NBSP);
+
 	uc = g_utf8_get_char (str);
-	while (uc != ' ' && uc != ENTITY_NBSP && offset) {
+	if (uc >= 0x80)
+		exec = FALSE;
+	while (exec && uc != ' ' && uc != ENTITY_NBSP && offset) {
 		str = g_utf8_prev_char (str);
 		uc = g_utf8_get_char (str);
+		if (uc >= 0x80)
+			exec = FALSE;
 		offset--;
 	}
 
@@ -1701,18 +1943,20 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 		offset++;
 	}
 
-	while (offset < text->text_len && !rv) {
-		for (i=0; i<MIM_N; i++) {
-			if (mim [i].preg && !regexec (mim [i].preg, str, 2, pmatch, 0)) {
-				paste_link (engine, text,
-					    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_so),
-					    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_eo), mim [i].prefix);
-				rv = TRUE;
-				break;
+	if (exec) {
+		while (offset < text->text_len && !rv) {
+			for (i=0; i<MIM_N; i++) {
+				if (mim [i].preg && !regexec (mim [i].preg, str, 2, pmatch, 0)) {
+					paste_link (engine, text,
+						    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_so),
+						    g_utf8_pointer_to_offset (text->text, str + pmatch [0].rm_eo), mim [i].prefix);
+					rv = TRUE;
+					break;
+				}
 			}
+			str = g_utf8_next_char (str);
+			offset++;
 		}
-		str = g_utf8_next_char (str);
-		offset++;
 	}
 
 	html_undo_level_end (engine->undo);
@@ -1738,8 +1982,8 @@ html_text_append (HTMLText *text, const gchar *str, gint len)
 	gchar *to_delete;
 
 	to_delete       = text->text;
+	text->text_len += text_len (&str, len);
 	text->text      = g_strconcat (to_delete, str, NULL);
-	text->text_len += text_len (str, len);
 
 	g_free (to_delete);
 
